﻿-- Current Memory Load:380
-- Quick Find Contents
-- -[SVARS]Saved Variables, Per Character
-- -[GVARS] Global Variables
--[[INITIALIZERS]]--
-- -[PLOAD] Loading Action
-- -[EVENT] Event Handler
-- -[UISET] Initial UIObject Setup
--[[UPDATERS]]--
-- -[UPDUI] Update UI Frame
-- -[UPDST] Update Skill Table
-- -[TIERS] Checks the index against the table to return the skill tier of a profession (i.e. Apprentice, master, etc.)
--[[COMMANDS]]--
-- -[SCMDS] /Commands
--[[PARSERS]]--
-- -[PARSE] Parser
-- -[MPARS] Multi-Parameter Command Parser
--[[CONFIG TOOLS]]--
-- -[CONFG]Sets the configuration mode of the frame as on/off
-- -[TCONF] Config mode toggle
--[[TRACKERS]]--
-- -[TRBAR] Sets a bars tracking to on or off
-- -[TBSET] command based bar tracking setter
-- -[TRALL] Set all bars to track
--[[FRAME MODIFIERS]]--
-- -[SCALE] Scales the Frame
-- -[ADJST] Adjusts the Frames Anchor after dragging to center based anchor
-- -[NUDGE] Move frame a small amount
-- -[RESET] Reset the UI
--[[UI INTERACTION]]--
-- -[HOVER] OnEnter/Onleave functions for Icons and Bars
--[[SUPPORT FUNCTIONS]]--
-- -[IDXOF] Returns the array index of the triggering item
-- -[PRINT] for quick printing from the XML

local ADDONNAME, PHUDdata = ...
local VERSION = GetAddOnMetadata(ADDONNAME,"Version")
PHUDdata.Functions = {}
local MyFuncs = PHUDdata.Functions;
local _;
--[SVARS]Saved Variables, Per Character
local defaults  = {
	PH_MODE 			= 0, 		-- 0:Short 		1:Long
	PH_LOCK 			= 0,		-- 0:unlocked	1:locked
	PH_VISIBLE 			= 1,		-- 0:not vis	1:vis
	PH_LINK 			= 1,		-- 0:plain		1:show links
	PH_EXPAND_DIRECTION = -1,		-- 1:up		   -1:down
	PH_FIRSTRUN 		= true,		-- if this is first run
	PH_CONFIGMODE 		= false,	-- in Config Mode?
	PH_SCALE = {					-- Archive the Frame Scale
			current 	= 1.00, 	-- current scale
			min 		= 0.50,		-- min scale
			max 		= 4.00		-- max scale
	},
	PH_LOCUS = {					-- Archive the Frame Location
			rp 			= "CENTER",	-- relativePoint
			x 			= 0,		-- xOffset
			y 			= 0			-- yOffset
	},
	PH_TRACKING = {					-- Default all On
			[1] 		= true,
			[2] 		= true,
			[3] 		= true,
			[4] 		= true,
			[5] 		= true,
			[6] 		= true
	},
}

-- LOCAL Variables
-- Text Color Constants
local White  				= "|cffffffff";
local Red    				= "|cffff0000";
local Green  				= "|cff00ff00";
local Blue   				= "|cff6060ff";
local Cyan   				= "|cff00ffff";
local Magenta				= "|cffff00ff";
local Yellow 				= "|cffffff00";
-- Bar State Colors
local SKILLMAXNEARCOLOR		= {r = 1.00, g = 0.00, b = 0.00};
local SKILLNORMALCOLOR		= {r = 0.25, g = 0.25, b = 0.75};
local SKILLCAPCOLOR 		= {r = 0.25, g = 0.65, b = 0.25};
-- 
local VariablesLoaded 		= false;
local LinkTemplate 			= "\124cffffd000\124Henchant:SPELL\124h[SKILL: NAME]\124h\124r"
local FrameUI 				= {Slider = nil, Edit = nil, XToggle = nil};
-- 
local SKILLCAPMSG 			= "Skill At Max Level";
local SKILLMAXMSG 			= "Seek Training";
-- Current Skill Cap
local SKILLCAP 				= 525--(GetAccountExpansionLevel() * 75) + 300;
-- UPValue our AddOn data (they're now private to our addon, not global to the WoW environment)
local CollectionTerm		= PHUDdata.CollectionTerm;
local Profession			= PHUDdata.Profession;
local ProfessionNameSlot	= PHUDdata.ProfessionNameSlot;
local UIObject 				= PHUDdata.UIObject;
local BarTexturePath		= PHUDdata.BarTexturePath;
local BarTextures			= PHUDdata.BarTextures;
local SkillTier				= PHUDdata.SkillTier;
PHUDdata.SkillName = {
		Mining 			= GetSpellInfo(2575),
		Toughness 		= GetSpellInfo(53120),
		Herbalism 		= PHUDdata.Localisation[GetLocale()].nameHerbalism,	-- no herbalism spell to reference...
		Lifeblood 		= GetSpellInfo(81708),
		Skinning 		= GetSpellInfo(8613),
		MasterOfAnatomy = GetSpellInfo(53125),
		Alchemy			= GetSpellInfo(2259),	
		ElixerMaster 	= GetSpellInfo(28677),
		PotionMaster 	= GetSpellInfo(28675),
		TransMaster 	= GetSpellInfo(28672),
		Engineering		= GetSpellInfo(4036),
		GbEngineer 		= GetSpellInfo(20222),
		GnEngineer 		= GetSpellInfo(20219),
		Insciption		= GetSpellInfo(45357),
		Milling 		= GetSpellInfo(51005),
		Jewelcrafting	= GetSpellInfo(25229),
		Prospecting 	= GetSpellInfo(31252),
		Tailoring		= GetSpellInfo(3908),
		ClothScav 		= GetSpellInfo(59390),
		Archaeology		= GetSpellInfo(78670),
		Survey 			= GetSpellInfo(80451),
		Cooking			= GetSpellInfo(2550),
		CookingFire 	= GetSpellInfo(818),
		Enchanting		= GetSpellInfo(7411),
		Disenchant 		= GetSpellInfo(13262),
		FirstAid		= GetSpellInfo(3273),
		Fishing			= GetSpellInfo(7620),
		Smelting		= GetSpellInfo(2656)
	}
local SkillName 			= PHUDdata.SkillName;

-- Slash Commands
SLASH_ProfessionHUD1, SLASH_ProfessionHUD2 = '/ph', '/phud';	-- /Commands

--[[INITIALIZERS]]--
--[PLOAD] Loading Action
function ProfessionHUD_OnLoad(self)
	self:SetBackdropBorderColor(TOOLTIP_DEFAULT_COLOR.r, TOOLTIP_DEFAULT_COLOR.g, TOOLTIP_DEFAULT_COLOR.b);
	self:SetBackdropColor(TOOLTIP_DEFAULT_BACKGROUND_COLOR.r, TOOLTIP_DEFAULT_BACKGROUND_COLOR.g, TOOLTIP_DEFAULT_BACKGROUND_COLOR.b);
	self:RegisterEvent("ADDON_LOADED");				--AddOn Loaded
	self:RegisterEvent("CHAT_MSG_SKILL");			--Skill Message Received
	self:RegisterEvent("CHAT_MSG_SYSTEM");			--Learn / Unlearn
	--self:RegisterEvent("PLAYER_TALENT_UPDATE");		
	self:RegisterEvent("LEARNED_SPELL_IN_TAB");
	self:RegisterEvent("SPELLS_CHANGED");
	self:RegisterEvent("TRAINER_UPDATE");
	self:RegisterEvent("SKILL_LINES_CHANGED");
	self:RegisterEvent("ADDON_ACTION_BLOCKED");
	self:RegisterEvent("ADDON_ACTION_FORBIDDEN");
end

--[EVENT] Event Handler
function ProfessionHUD_OnEvent(self, event, ...)
	-- Starts the addon
	if (event == "ADDON_LOADED") then
		if (...) == ADDONNAME then
			DEFAULT_CHAT_FRAME:AddMessage(Green..ADDONNAME.." "..Red..VERSION..Green..": Loaded. Type "..Cyan.."/ph"..Green.." for help.");
			ProfessionHUDPC = ProfessionHUDPC or CopyTable(defaults) -- load our defaults if we have no SV yet (first run)
			PHLocalisation = PHUDdata.Localisation[GetLocale()];
			SLASH_ProfessionHUD3 = PHLocalisation.slashCommand;
			MyFuncs.PH_Initialize();
			VariablesLoaded = true;
			MyFuncs.PH_UpdateUI();
		end
	-- Fires on Skill Up
	elseif (event == "CHAT_MSG_SKILL" or event == "CHAT_MSG_SYSTEM") then
		msg = ...
		MyFuncs.PH_UpdateUI();
		MyFuncs.PH_Parser(msg, event);
		
	-- Fires on other update events
	elseif (event == "SPELLS_CHANGED" or 
			event == "LEARNED_SPELL_IN_TAB" or 
			event == "TRAINER_UPDATE" or
			event == "SKILL_LINES_CHANGED" --or 
			--event == "PLAYER_TALENT_UPDATE" 
			) then
		--Temporary fix for glyph interference
		if((IsAddOnLoaded("Blizzard_TalentUI") or IsAddOnLoaded("Blizzard_GlyphUI")) and PlayerTalentFrame:IsShown()) then
			return
		end
		MyFuncs.PH_UpdateUI();
	--elseif (event == "ADDON_ACTION_BLOCKED" or event == "ADDON_ACTION_FORBIDDEN") then
	--	msg = ...
	--	print("PH:Error: "..event);
	end
end

--[UISET] Initial UIObject Setup
function MyFuncs.PH_Initialize()
	
	for i =1,6 do	
		UIObject[i].bar = CreateFrame("StatusBar", nil, PHUD, "BarTemplate");
		UIObject[i].bar:SetPoint("TOPLEFT", PHUD, "TOPLEFT", 53, -(i * 23 - 14));
		UIObject[i].icon1 =  CreateFrame("Button", nil, UIObject[i].bar, "IconTemplate");
		UIObject[i].icon1:SetPoint("RIGHT", UIObject[i].bar, "LEFT", -5, 0);
		UIObject[i].icon2 =  CreateFrame("Button", nil, UIObject[i].icon1, "IconTemplate");
		UIObject[i].icon2:SetPoint("RIGHT", UIObject[i].icon1, "LEFT", -2, 0);
		UIObject[i].check = CreateFrame("CheckButton", nil, UIObject[i].bar, "CheckTemplate");
		UIObject[i].check:SetPoint("Left", UIObject[i].bar, "RIGHT");
		UIObject[i].bar.Text:SetText("["..PHLocalisation.msgNotLearned.."]");
	end
	FrameUI.XToggle = CreateFrame("Button", nil, PHUD, "RedButtonTemplate");
	FrameUI.XToggle:SetPoint("BOTTOM", PHUD, "BOTTOM", 0, 65)
	FrameUI.XToggle:SetFrameStrata("HIGH");
	FrameUI.Slider = CreateFrame("Slider", "ScaleSlide", PHUD, "SliderTemplate");
	FrameUI.Slider:SetPoint("BOTTOM", PHUD, "BOTTOM", -20, 15);
	FrameUI.Slider:SetMinMaxValues(ProfessionHUDPC.PH_SCALE.min, ProfessionHUDPC.PH_SCALE.max);
	FrameUI.Edit = CreateFrame("EditBox", "ScaleEdit", PHUD, "EditTemplate");
	FrameUI.Edit:SetPoint("BOTTOMRIGHT", PHUD, "BOTTOMRIGHT", -5, 20);
	PHUD:SetPoint("CENTER", UIParent, ProfessionHUDPC.PH_LOCUS.rp, ProfessionHUDPC.PH_LOCUS.x, ProfessionHUDPC.PH_LOCUS.y);
	MyFuncs.PH_SetScale(ProfessionHUDPC.PH_SCALE.current);
	MyFuncs.PH_SetConfigMode(false);
	MyFuncs.PH_Adjust();
end

--[[UPDATERS]]--
--[UPDUI] Update UI Frame
function MyFuncs.PH_UpdateUI()
	if(not VariablesLoaded) then
		return
	end
	MyFuncs.PH_UpdateTable();
	local counter = 0;
	for i = 1, 6 do
		if(Profession[i].id ~= nil and Profession[i].id ~= 0) then
			--Setup Icon 1
			UIObject[i].icon1:Show();
			UIObject[i].icon1:SetBackdrop({bgFile=UIObject[i].tex1});
			UIObject[i].icon1:SetAttribute("type", "spell");
			--redirect mining link to smelting
			if(Profession[i].name1 == SkillName.Mining) then
				UIObject[i].icon1:SetAttribute("spell", SkillName.Smelting);
			else
				UIObject[i].icon1:SetAttribute("spell", Profession[i].name1);
			end
			local _,_,_,_, numskills = GetProfessionInfo(Profession[i].id)
			--Setup Icon2 if available
			if (numskills == 2) then
				UIObject[i].icon2:Show();
				UIObject[i].icon2:SetBackdrop({bgFile=UIObject[i].tex2});
				UIObject[i].icon2:SetAttribute("type", "spell");
				UIObject[i].icon2:SetAttribute("spell", Profession[i].name2);
			else
				UIObject[i].icon2:Hide();
			end
			--Set Bar Values
			UIObject[i].bar:SetMinMaxValues(0,Profession[i].max);
			UIObject[i].bar:SetValue(Profession[i].rank);
			--Format Bar Color
			if(Profession[i].rank >= Profession[i].max - 25 and Profession[i].rank < Profession[i].cap and Profession[i].max < Profession[i].cap) then
				UIObject[i].bar:SetStatusBarColor(SKILLMAXNEARCOLOR.r, SKILLMAXNEARCOLOR.g, SKILLMAXNEARCOLOR.b);
			elseif (Profession[i].rank == Profession[i].cap) then
				UIObject[i].bar:SetStatusBarColor(SKILLCAPCOLOR.r, SKILLCAPCOLOR.g, SKILLCAPCOLOR.b);
			else
				UIObject[i].bar:SetStatusBarColor(SKILLNORMALCOLOR.r, SKILLNORMALCOLOR.g, SKILLNORMALCOLOR.b);
			end
			--Format Bar Text
			local _,_,_,_,_,_,_,modifier = GetProfessionInfo(Profession[i].id);
			local modText = "";
			if(modifier ~= nil and modifier > 0) then
				modText = Green.."+"..modifier..White;
			end
			if(ProfessionHUDPC.PH_MODE == 0) then
				UIObject[i].bar.Text:SetText(Profession[i].name1);
			else
				UIObject[i].bar.Text:SetText(Profession[i].name1.." ["..Profession[i].rank..modText.."/"..Profession[i].max.."]");	
			end
			MyFuncs.PH_TrackBar(i, true);
			counter = counter + 1;
		else
			local _, max = UIObject[i].bar:GetMinMaxValues();
			MyFuncs.PH_TrackBar(i, false);
			UIObject[i].bar.Text:SetText("["..PHLocalisation.msgNotLearned.."]");
			UIObject[i].bar:SetStatusBarColor(SKILLNORMALCOLOR.r, SKILLNORMALCOLOR.g, SKILLNORMALCOLOR.b);
			UIObject[i].bar:SetValue(max);
			UIObject[i].icon1:Hide();
			UIObject[i].icon2:Hide(); 			
		end
	end
	--Adjust visibility if not visible or has no bars
	if (counter == 0 or ProfessionHUDPC.PH_VISIBLE == 0) then
		PHUD:Hide();
	else
		PHUD:Show();
	end
end

--[UPDST] Update Skill Table
function MyFuncs.PH_UpdateTable()
	for i = 1,6 do
		--Searches for reversed skills(i.e. skill is top button, buff is bottom)
		local text1 = ProfessionNameSlot[i]:GetText()
		local text2 = ProfessionNameSlot[i + 6]:GetText()
		if (text2 == SkillName.Skinning) then
			text2 = text1
			text1 = SkillName.Skinning
		end
		if (text1 ~= nil) then --Gets spell information from name
			Profession[i].name1,_,UIObject[i].tex1 = GetSpellInfo(text1)
		end
		
		if (text2 ~= nil) then --Gets spell information from name
			Profession[i].name2,_,UIObject[i].tex2 = GetSpellInfo(text2)
		end
	end
	
	Profession[1].id, Profession[2].id, Profession[3].id, Profession[4].id, Profession[5].id, Profession[6].id= GetProfessions()
	for i = 1, 6 do
		if(Profession[i].id ~= nil and Profession[i].id ~= 0) then
			local numskills
			Profession[i].lastRank = Profession[i].rank;
			Profession[i].name1, UIObject[i].tex1, Profession[i].rank, Profession[i].max, numskills = GetProfessionInfo(Profession[i].id);
			Profession[i].spell1 = MyFuncs.PH_GetSkillTier(Profession[i].name1)
			Profession[i].cap = MyFuncs.SetSkillCap(Profession[i].name1);
			if (numskills == 2) then
				Profession[i].spell2 = MyFuncs.PH_GetSkillTier(Profession[i].name2)
			end
		else
			Profession[i].name1, Profession[i].rank, Profession[i].max, Profession[i].spell1, Profession[i].name2, Profession[i].spell2 = 0, 0, 0, 0, 0, 0
		end
	end
end

--[TIERS] Checks the index against the table to return the skill tier of a profession (i.e. Apprentice, master, etc.)
function MyFuncs.PH_GetSkillTier(name)
	local tier = 0
	local index = 0
	local prof = nil
	--convert the passed-in name to either a end skillId or a profession parent name
	if (name == SkillName.Toughness) then 
		prof = SkillName.Mining
	elseif (name == SkillName.Lifeblood) then
		prof = SkillName.Herbalism
	elseif (name == SkillName.MasterOfAnatomy) then
		prof = SkillName.Skinning
	elseif (name == SkillName.ElixerMaster)then
		return 28677
	elseif (name == SkillName.PotionMaster)then
		return 28675
	elseif (name == SkillName.TransMaster)	then
		return 28672
	elseif (name == SkillName.GbEngineer)then
		return 20222
	elseif (name == SkillName.GnEngineer)then
		return 20219
	elseif (name == SkillName.Milling)then
		return 51005
	elseif (name == SkillName.Prospecting)then
		return 31252
	elseif (name == SkillName.ClothScav)then
		return 59390
	elseif (name == SkillName.Survey)then
		return 80451
	elseif (name == SkillName.CookingFire)then
		return 818
	elseif (name == SkillName.Disenchant) then
		return 13262
	elseif (name == nil or name == 0) then
		return -1
	end
	for i = 1, 6 do
		if(Profession[i].name1 == name or Profession[i].name1 == prof or Profession[i].name2 == name or Profession[i].name2 == prof) then
				index = i
				break
		end
	end
	for i = 1, 8 do
		if (name == SkillName.Lifeblood) then
			if(Profession[index].rank >= ((i - 1) * 75)) then
				tier = i
			end
		elseif (name == SkillName.Toughness or name == SkillName.MasterOfAnatomy) then
			if(Profession[index].rank >= (i * 75)) then
				tier = i
			end
		else
			if ((Profession[index].max >= (i * 75)) and (Profession[index].max <= (i * 75) + 15)) then
				tier = i
				break
			end
		end
	end
	return SkillTier[name][tier]
end

--[[COMMANDS]]--
--[SCMDS] /Commands
function SlashCmdList.ProfessionHUD(msg)
	local param = {};
	msg, param = MyFuncs.PH_MultCmdParse(msg);
	local later = Green.."PHUD:"..Red.." This Command Will Be Active Later";
	if (msg == 'mode' or msg =='toggle') then 
		if (ProfessionHUDPC.PH_MODE == 0) then 
			ProfessionHUDPC.PH_MODE = 1;
			print(Green.."PHUD:"..Cyan.." Long Mode On.");
		else
			ProfessionHUDPC.PH_MODE = 0
			print(Green.."PHUD:"..Cyan.." Short Mode On.");
		end
		MyFuncs.PH_UpdateUI();
	elseif (msg == 'hide') then
		print(Green.."PHUD:"..Cyan.." Hiding.");
		MyFuncs.PH_SetConfigMode(false);
		PHUD:Hide();
		ProfessionHUDPC.PH_VISIBLE = 0;
	elseif (msg == 'show') then
		local learned, tracking = 6, 6;
		for i = 1,6 do
			if (Profession[i].id == nil) then
				learned = learned - 1;
			end
			if (ProfessionHUDPC.PH_TRACKING[i] == false) then
				tracking = tracking - 1;
			end
		end
		if (learned == 0) then
			print(Green.."PHUD:"..Red.." No Skills To Show.");
		elseif (tracking == 0) then
			print(Green.."PHUD:"..Red.." No Skills Being Tracked. Switching To Config Mode.");
			MyFuncs.PH_SetConfigMode(true);
		else
			ProfessionHUDPC.PH_VISIBLE = 1;
			print(Green.."PHUD:"..Cyan.." Showing.");
			PHUD:Show();
		end
	elseif (msg == 'lock') then
		ProfessionHUDPC.PH_LOCK = 1;
		print(Green.."PHUD:"..Cyan.." Frame Locked.");
		if (ProfessionHUDPC.PH_CONFIGMODE) then
			return
		end
		local wd,ht = PHUD:GetSize();
		if (wd > 224) then
			wd = 224;
			MyFuncs.PH_Nudge(-13, 0);
		end
		PHUD:SetSize(wd, ht);
		PHUDFrame:StopMovingOrSizing();
	elseif (msg == 'unlock') then
		ProfessionHUDPC.PH_LOCK = 0
		print(Green.."PHUD:"..Cyan.." Frame Unlocked.");
	elseif (msg == 'prime' or msg == 'primary' or msg == 'pri') then
		print(Green.."PHUD:"..Cyan.." Showing Only Primary Skills.");
		PH_SetBar(msg, true);
	elseif (msg == 'sec' or msg== 'secondary') then
		print(Green.."PHUD:"..Cyan.." Showing Only Secondary Skills.");
		PH_SetBar(msg, true);
	elseif (msg == 'all') then
		print(Green.."PHUD:"..Cyan.." Showing All Skills.");
		PH_SetBar(msg, true);
	elseif (msg == 'reset') then
		print(Green.."PHUD:"..Cyan.." Resetting.");
		MyFuncs.PH_Reset();
	elseif (msg == 'where') then
		print("["..Red.."Debug"..White.."] ");
		print(PHUD:GetPoint());
		print(PHUD:GetScale());
	elseif (msg == 'move') then
		local p, rt, rp, x, y = param[0], param[1], param[2], param[3], param[4];
		print(p..","..rt..","..rp..","..x..","..y);
		PHUD:ClearAllPoints();
		PHUD:SetPoint(p, rt, rp, x, y);
	elseif (msg == 'link') then
		print(Green.."PHUD:"..Cyan.." Attaching "..Yellow.."[Links]"..Cyan..".");
		ProfessionHUDPC.PH_LINK = 1;
	elseif (msg == 'plain') then 
		print(Green.."PHUD:"..Cyan.." Detaching "..White.."Links"..Cyan..".");
		ProfessionHUDPC.PH_LINK = 0;
	elseif (msg == 'config' or msg == 'cf' or msg == PHLocalisation.cmdConfig) then
		MyFuncs.PH_ToggleConfig();
	elseif (msg == 'refresh' or msg == 'rf') then
		MyFuncs.PH_UpdateUI();
	elseif (msg == 'g' or msg == 'guide' or msg == 'p1') then
		print(" ")
		print(Green..ADDONNAME.." "..VERSION..Cyan.." New User's Guide")
		print(Cyan.."Somewhere on your screen, default at center, you will see the ProfessionHUD(PH) Frame.")
		print(Cyan.."This will show your currently known "..Green.."Professions"..Cyan..".")
		print(Cyan.."If you move your mouse over a bar, it will show your current and max skill.")
		print(Cyan.."Typing "..Green.."/ph mode"..Cyan.." or "..Green.."/ph toggle"..Cyan.." will show or hide the skill numbers.")
		print(Cyan.."While in Long Mode, putting your mouse over a bar will show your remaining points until your current maximum.")
		print(Yellow.."Type "..Green.."/ph p2"..Yellow.." for more.")
	elseif (msg == 'p2') then
		print(" ")
		print(Cyan.."Typing "..Green.."/ph unlock"..Cyan.." will allow you to click and drag the frame to anywhere you like.")
		print(Cyan.."Typing "..Green.."/ph lock"..Cyan.." will lock it in place.")
		print(Cyan.."Typing "..Green.."/ph hide"..Cyan.." or "..Green.."/ph show"..Cyan.." will hide and show the frame, respectively. The frame will not show if there are no skills to show.")
		print(Cyan.."The Icons beside the PH bars symbolize your "..Green.."Professions"..Cyan..". Clicking them will open the skill pane for that "..Green.."Profession"..Cyan..".")
		print(Yellow.."Type "..Green.."/ph p3"..Yellow.." for more.")
	elseif (msg == 'p3') then
		print(" ")
		print(Cyan.."When you "..Blue.."incease in a skill"..Cyan..", I will update the bars.")
		print(Cyan.."If your skill in a "..Green.."Profession"..Cyan.." raises to where you can learn a new skill "..Yellow.."from a trainer"..Cyan..", then I will let you know.")
		print(Cyan.."By default, I will "..Yellow.."link"..Cyan.." the new skills to your chat window.")
		print(Cyan.."Typing "..Green.."/ph plain"..Cyan.." will send the skill names in "..White.."plain text"..Cyan..".")
		print(Cyan.."Typing "..Green.."/ph link"..Cyan.." will return to "..Yellow.."link"..Cyan.."ed mode.")
		print(Yellow.."This concludes the guide. Have fun!")
	elseif (msg == 'track') then
		MyFuncs.PH_TrackBarSet(param[0], true);
	elseif (msg == 'untrack') then
		MyFuncs.PH_TrackBarSet(param[0], false);
	elseif (msg == 'xdir') then
		if (param[0] == 'up') then 
			ProfessionHUDPC.PH_EXPAND_DIRECTION = 1;
		else
			ProfessionHUDPC.PH_EXPAND_DIRECTION = -1;
		end
	elseif (msg == 'test') then
		-- local number = tonumber(param[0]);
		-- for i = 1, 6 do
			-- PHUDdata.UIObject[i].bar:SetStatusBarTexture(PHUDdata.BarTexturePath..PHUDdata.BarTexture[number]);
		-- end
		MyFuncs.SetModifier();
	else
		print("PHUD: Usage");
		print(Green.."/ph "..Cyan.."[arg]");
		print(Green.."/ProfessionHUD "..Cyan.."[arg]");
		print(Yellow.."Args:");
		print(Cyan.." [g | guide] (Show An Explanation Of Features)");
		print(Green.." [help"..White.." or "..Cyan.. "[no arg]]"..White.." (Show This Menu)");
		print(Green.." [config"..White.." or "..Cyan.. "cf]"..White.." (Toggle Config Mode)");
		print(Green.." [refresh"..White.." or "..Cyan.. "rf]"..White.." (Refresh Frame)");
		print(Green.." [mode | toggle] "..White.."(Notation Mode Toggle)");
		print(Green.." [hide | show] "..White.."(Hide/Show Frame)");
		print(Green.." [lock | unlock] "..White.."(Lock/Unlock Frame)");
		print(Green.." [link | plain] "..White.."(Show New Skills As Links/Plain Text)");
		print(Green.." [track | untrack] #"..White.."(Tracks/Untracks the #th bar Of The 6)");
		print(Green.." [track inc] "..White.."(Tracks Only Incomplete Skills)");
		print(Green.." [reset] "..White.."(Resets Frame To Center)");
		print(Green.." [pri | prime | primary] "..White.."(Show Primary Skills Only)");
		print(Green.." [sec | secondary] "..White.."(Show Secondary Skills Only)");
		print(Green.." [all] "..White.."(Show All Skills)");
		print(Green.." [xdir up | down] "..White.."(Sets Frame Expansion Direction)");
		print(Red.."[NI]"..Green.." = Not Implemented.");
		print(Green.."End Of Commands");
	end
end

--[[PARSERS]]--
--[PARSE] Parser
function MyFuncs.PH_Parser(msg, event)
	--Parse Skill Message
	local sub, find, len = string.sub, string.find, string.len
	if(event == "CHAT_MSG_SYSTEM") then
		--Unlearned a skill
		if (find(msg,PHLocalisation.msgUnlearn) ~= nil) then 
			if (not ProfessionHUDPC.PH_CONFIGMODE) then
				MyFuncs.PH_Nudge(0, -3.85 * ProfessionHUDPC.PH_EXPAND_DIRECTION);
			end
			MyFuncs.PH_SetConfigMode(ProfessionHUDPC.PH_CONFIGMODE);
			return;
		end
	elseif(event == "CHAT_MSG_SKILL") then
		--ignore riding/runeforging
		local riding = PHLocalisation.nameRiding;
		local rforging = GetSpellInfo(53428);
		if (find(msg,riding) ~= nil or find(msg,rforging) ~= nil) then
			return; 
		end
		--Skill Learned
		if (find(msg,PHLocalisation.msgLearn) ~= nil) then
			--get the skill name
			learnedName = MyFuncs.GetSkillName(msg);
			if (learnedName == nil) then 
				return 
			end
			--Clear the secondary icon in preparation
			for i = 1, 6 do
				if(learnedName == Profession[i].name1) then
					MyFuncs.PH_TrackBar(i, true);
				end
				UIObject[i].icon2:SetBackdrop({bgFile=nil});
			end
			if (not ProfessionHUDPC.PH_CONFIGMODE) then
				MyFuncs.PH_Nudge(0, 11.5 * ProfessionHUDPC.PH_EXPAND_DIRECTION);
			end
			MyFuncs.PH_UpdateUI()
			MyFuncs.PH_SetConfigMode(ProfessionHUDPC.PH_CONFIGMODE);
			return;
		end
		local skillIndex = 0;
		for i = 1, 6 do
			if(find(msg,Profession[i].name1) ~= nil) and (Profession[i].name1 ~= 0) then
				skillIndex = i;
				break;
			end
		end
		if (Profession[skillIndex].name1 == SkillName.Fishing) then
			return --dont do anything, yet
		end
		--Get relevant chart data
		local chart = PHUDdata.Charts[Profession[skillIndex].name1];
		local chartSize = PHUDdata.Sizes[chart];
		--Override for skinning
		if (Profession[skillIndex].name1 == SkillName.Skinning) then
			if (Profession[skillIndex].rank < 100) then
				if ((Profession[skillIndex].rank / 10) == floor(Profession[skillIndex].rank/10))then
					Profession[skillIndex].rank = floor((Profession[skillIndex].rank / 10) + 10)
					print(Green..CollectionTerm[Profession[skillIndex].name1]..Yellow..Profession[skillIndex].rank);
					return
				end
			else
				if ((Profession[skillIndex].rank / 5) == floor(Profession[skillIndex].rank/5))then
					Profession[skillIndex].rank = floor(Profession[skillIndex].rank / 5)
					print(Green..CollectionTerm[Profession[skillIndex].name1]..Yellow..Profession[skillIndex].rank)
					return
				end
			end
			return;
		end
		--Print all nodes that match new rank
		local first = 0
		local swap = string.gsub
		for i = 1, chartSize do
			if (chart[i].Level > Profession[skillIndex].rank) then
				break
			end
			
			for j = Profession[skillIndex].lastRank + 1, Profession[skillIndex].rank do
				if(chart[i].Level == j) then
					if (first == 0) then
						print(Blue..CollectionTerm[Profession[skillIndex].name1]) --Post the "You may now do.." message
						first = 1
					end
					if (Profession[skillIndex].name1 == SkillName.Mining or 
						Profession[skillIndex].name1 == SkillName.Herbalism or 
						Profession[skillIndex].name1 == SkillName.Skinning) 
						then
						print(chart[i].Name)
					elseif (ProfessionHUDPC.PH_LINK == 0) then
						local name = GetSpellInfo(chart[i].Spell)
						print(name)
					else 
						local message = LinkTemplate
						local name = GetSpellInfo(chart[i].Spell)
						message = swap(message, ("SPELL"), chart[i].Spell)
						message = swap(message, ("SKILL"), Profession[skillIndex].name1)
						message = swap(message, ("NAME"), name)
						print(message)
					end
				elseif(chart[i].Level > Profession[skillIndex].rank) then 
					return;
				end
			end
		end
	end
end

--[MPARS] Multi-Parameter Command Parser
function MyFuncs.PH_MultCmdParse(msg)
	--handle empty msg
	if (msg == "") then
		return "";
	end
	local pos, size, main;
	local sub, find, len = string.sub, string.find, string.len;
	pos = find(msg, " ");
	if (pos == nil) then
		return msg; --handle single param commands
	end
	main = sub(msg, 0, pos - 1);
	size = len(msg);
	msg = sub(msg, pos + 1, size);
	local i = 0;
	local param = {};
	while ((msg ~= "") and (msg ~= " ") and (i < 30)) do
		pos = find(msg, " ");
		if (pos == nil) then
			param[i] = msg;
			break;
		else
			param[i] = sub(msg, 0, pos - 1);
			msg = sub(msg, pos + 1, size);
		end
		i = i + 1;
	end;
	return main, param;
end

--[[CONFIG TOOLS]]--
--[CONFG]Sets the configuration mode of the frame as on/off
function MyFuncs.PH_SetConfigMode(configMode)
	local configHeight = 230;
	MyFuncs.PH_UpdateTable();
	local count = 0;
	local wd, ht = PHUD:GetSize();
	if(configMode == true) then --Config Mode	input:true
		PHUD:Show();
		FrameUI.Slider:Show();
		FrameUI.Edit:Show();
		FrameUI.XToggle:Show();
		for i = 1, 6 do
			UIObject[i].bar:Show();
			UIObject[i].bar:SetPoint("TOPLEFT", PHUD, "TOPLEFT", 53, -(i * 23 - 14));
			UIObject[i].check:Show();
			MyFuncs.PH_TrackBar(i, ProfessionHUDPC.PH_TRACKING[i]);
		end
		if (not ProfessionHUDPC.PH_CONFIGMODE) then
			MyFuncs.PH_Nudge(10.5, (ht - configHeight) / 2);
		end
		PHUD:SetSize(245, configHeight);
		ProfessionHUDPC.PH_CONFIGMODE = true;
	else						--Op Mode		input:false
		FrameUI.Slider:Hide();
		FrameUI.Edit:Hide();
		FrameUI.XToggle:Hide();
		for i = 1, 6 do
			if(ProfessionHUDPC.PH_TRACKING[i]) then
				count = count + 1;
				UIObject[i].bar:Show();
				UIObject[i].bar:SetPoint("TOPLEFT", PHUD, "TOPLEFT", 53, -(count * 23 - 14));
			else
				UIObject[i].bar:Hide();
			end
			UIObject[i].check:Hide();
		end
		if(count == 0) then 
			PHUD:Hide();
		else
			--PHUD:Show();
		end
		local yNew = (count * 23) + 11;
		if (ProfessionHUDPC.PH_CONFIGMODE) then
			MyFuncs.PH_Nudge(-10.5, (ht - yNew) / 2);
		end
		PHUD:SetSize(224, yNew);
		ProfessionHUDPC.PH_CONFIGMODE = false;
	end;
	MyFuncs.PH_UpdateUI();
end

--[TCONF] Config mode toggle
function MyFuncs.PH_ToggleConfig()
	MyFuncs.PH_SetConfigMode(not ProfessionHUDPC.PH_CONFIGMODE)
end

--[[TRACKERS]]--
--[TRBAR] Sets a bars tracking to on or off
function MyFuncs.PH_TrackBar(index, track)
	if(not VariablesLoaded) then
		return;
	end
	if (track ~= true and track ~= false and track ~= nil and track ~= 1) then
		return;
	elseif (track == 1) then
		track = true;
	elseif (track == nil) then
		track = false;
	end
	--Decide the alpha level
	local alpha = 1;
	if(track) then
		--Lock Unlearned skills from being tracked
		if (Profession[index].id == nil or Profession[index].id < 1) then
			UIObject[index].check:SetChecked(0);
			return;
		end
		alpha = 1;
		UIObject[index].check:SetChecked(1);
	else
		alpha = .2;
		UIObject[index].check:SetChecked(0);
	end
	ProfessionHUDPC.PH_TRACKING[index] = track;
	--Set alpha
	UIObject[index].bar:SetAlpha(alpha);
end

--[TBSET] command based bar tracking setter
function MyFuncs.PH_TrackBarSet(msg, track)
	if(not VariablesLoaded) then
		return;
	end
	local list = ProfessionHUDPC.PH_TRACKING
	--check for user input number
	for i = 1, 6 do
		if(tonumber(msg) == i) then --is a number
			list[i] = track;
		elseif(msg == 'inc' or msg == 'incomplete') then
			if(Profession[i].id ~= nil and Profession[i].rank < 525) then
				list[i] = track;
			else 
				list[i] = not track;
			end;
		end
	end
	if(msg == 'pri' or msg == 'prime' or msg == 'primary') then
		list[1], list[2] = track, track;
	elseif (msg == 'sec' or msg == 'secondary') then
		list[3], list[4], list[5], list[6] = track, track, track, track;
	elseif (msg == 'all') then
		list[1], list[2], list[3], list[4], list[5], list[6] = track, track, track, track, track, track;
	end
	for i = 1, 6 do
		if(Profession[i].id ~= nil) then
			MyFuncs.PH_TrackBar(i, list[i]);
			UIObject[i].check:SetChecked(list[i]);
		end
	end	
	MyFuncs.PH_SetConfigMode(false);
	MyFuncs.PH_UpdateUI();
end

--[TRALL] Set all bars to track
function MyFuncs.PH_TrackAll()
	if(not VariablesLoaded) then
		return;
	end
	for i = 1, 6 do
		if(Profession[i].id ~= nil and Profession[i].id > 0) then
			ProfessionHUDPC.PH_TRACKING[i] = true;
			MyFuncs.PH_TrackBar(i, true);
			UIObject[i].check:SetChecked(1);
		else
			ProfessionHUDPC.PH_TRACKING[i] = false;
			MyFuncs.PH_TrackBar(i, false);
			UIObject[i].check:SetChecked(0);
		end
	end
	MyFuncs.PH_SetConfigMode(false);
end

--[[FRAME MODIFIERS]]--
--[SCALE] Scales the Frame
function MyFuncs.PH_SetScale(scale)
	--clamp scale
	if (scale > ProfessionHUDPC.PH_SCALE.max) then
		scale = ProfessionHUDPC.PH_SCALE.max;
	elseif (scale < ProfessionHUDPC.PH_SCALE.min) then
		scale = ProfessionHUDPC.PH_SCALE.min;
	end
	--update location to remain centered at current location
	local _, _, rp, x, y = PHUD:GetPoint();
	y = y * ProfessionHUDPC.PH_SCALE.current / scale;
	x = x * ProfessionHUDPC.PH_SCALE.current / scale;
	--update scale
	ProfessionHUDPC.PH_SCALE.current = scale;
	PHUD:SetScale(scale);
	--update anchor
	PHUD:ClearAllPoints();
	PHUD:SetPoint("CENTER", UIParent, rp, x, y)
	ProfessionHUDPC.PH_LOCUS.rp, ProfessionHUDPC.PH_LOCUS.x, ProfessionHUDPC.PH_LOCUS.y = rp, x, y;
	--update UI controls
	FrameUI.XToggle:SetScale(scale*.75);
	FrameUI.Slider:SetValue(scale);
	_G[FrameUI.Slider:GetName()..'Text']:SetText(PHLocalisation.tagScale..string.format("%.2f", scale));
	FrameUI.Edit:SetText(string.format("%.2f", scale));
end

--[ADJST] Adjusts the Frames Anchor after dragging to center based anchor
function MyFuncs.PH_Adjust()
	local p, _, rp, x, y = PHUD:GetPoint();
	local wd, ht = PHUD:GetSize();
	wd, ht = wd/2, ht/2;
	if ((rp == "TOP") or (rp == "TOPLEFT") or (rp == "TOPRIGHT")) then
		y = y - ht;
	elseif ((rp == "BOTTOM") or (rp == "BOTTOMLEFT") or (rp == "BOTTOMRIGHT")) then
		y = y + ht;
	end
	if ((rp == "LEFT") or (rp == "TOPLEFT") or (rp == "BOTTOMLEFT")) then
		x = x + wd;
	elseif ((rp == "RIGHT") or (rp == "TOPRIGHT") or (rp == "BOTTOMRIGHT")) then
		x = x - wd;
	end
	PHUD:ClearAllPoints();
	PHUD:SetPoint("CENTER", UIParent, rp, x, y);
end

--[NUDGE] Move frame a small amount
function MyFuncs.PH_Nudge(xNew, yNew)
	local _,_, rp, x, y = PHUD:GetPoint();
	x, y = x + xNew, y + yNew;
	PHUD:ClearAllPoints();
	PHUD:SetPoint("CENTER", UIParent, rp, x, y);
end

--[RESET] Reset the UI
function MyFuncs.PH_Reset()
	ProfessionHUDPC = CopyTable(defaults);
	PHLocalisation = PHUDdata.Localisation[GetLocale()];
	PHUD:ClearAllPoints();
	PHUD:SetPoint("CENTER", "UIParent", "CENTER", 0, 0);
	if(VariablesLoaded) then
		MyFuncs.PH_TrackAll();	
	end
	MyFuncs.PH_SetConfigMode(false);
end

--[BRTEX] Bar Texture Modifier
function MyFuncs.PH_SetBarTexture(name)for i = 1, 6 do
		UIObject[i].bar:SetStatusBarTexture(BarTexturePath..name)
	end
end

--[[UI INTERACTION]]--
--[HOVER] OnEnter/Onleave functions for Icons and Bars
--mouse entering bar
function MyFuncs.PH_BarOnEnter(index)
	if ((Profession[index].id ~= nil) and (Profession[index].id ~= 0)) then
		if (ProfessionHUDPC.PH_MODE == 0)then
			UIObject[index].bar.Text:SetText("["..Profession[index].rank.."/"..Profession[index].max.."]");
		else 
			if(Profession[index].rank == SKILLCAP) then
				UIObject[index].bar.Text:SetText(SKILLCAPMSG);
			elseif (Profession[index].rank == Profession[index].max) then
				UIObject[index].bar.Text:SetText(SKILLMAXMSG);
			else
				UIObject[index].bar.Text:SetText(Profession[index].max - Profession[index].rank..PHLocalisation.tagPointsLeft);
			end
		end
	end
end

--mouse leaving bar
function MyFuncs.PH_BarOnLeave(index)
	if ((Profession[index].id ~= nil) and (Profession[index].id ~= 0)) then
		local _,_,_,_,_,_,_,modifier = GetProfessionInfo(Profession[index].id);
		local modText = "";
		if(modifier ~= nil and modifier > 0) then
			modText = Green.."+"..modifier..White;
		end
		if(ProfessionHUDPC.PH_MODE == 0) then
			UIObject[index].bar.Text:SetText(Profession[index].name1);
		else
			UIObject[index].bar.Text:SetText(Profession[index].name1.." ["..Profession[index].rank..modText.."/"..Profession[index].max.."]");	
		end
	end
end

--mouse entering Icon
function MyFuncs.PH_IconOnEnter(index1, index2)
	GameTooltip_SetDefaultAnchor(GameTooltip, UIParent)
	local name;
	if (index2 == 1) then
		name = Profession[index1].name1;
	else
		name = Profession[index1].name2;
	end
	GameTooltip:SetSpellByID(MyFuncs.PH_GetSkillTier(name));
	GameTooltip:Show()
end

--mouse leaving icon
function MyFuncs.PH_IconOnLeave()
	GameTooltip:Hide()
end

--[[SUPPORT FUNCTIONS]]--
--[IDXOF] Returns the array index of the triggering item
function MyFuncs.PH_IndexOf(self)
	for i =1,6 do
		if (self == UIObject[i].bar) or (self == UIObject[i].check) then
			return i;
		elseif (self == UIObject[i].icon1) then 
			return i, 1;
		elseif (self == UIObject[i].icon2) then
			return i, 2;
		end
	end
	return -1;
end

--[PRINT] for quick printing from the XML
function MyFuncs.PH_Print(msg)
	print(msg);
end

--[SSCAP] set the modified racial skill cap.
function MyFuncs.SetSkillCap(skill)
	local _,raceID = UnitRace("player")
	local modifier = 0
	if(raceID == "Tauren" and skill == SkillName.Herbalism) then
		modifier = 15;
	--elseif(raceID == "Dwarf" and skill == SkillName.Archaeology) then
	--	modifier = 15;
	elseif(raceID == "Gnome" and skill == SkillName.Engineering) then
		modifier = 15;
	elseif(raceID == "Draenei" and skill == SkillName.Jewelcrafting) then
		modifier = 10;
	elseif(raceID == "Worgen" and skill == SkillName.Skinning) then
		modifier = 15;
	elseif(raceID == "Bloodelf" and skill == SkillName.Enchanting) then
		modifier = 10;
	elseif(raceID == "Goblin" and skill == SkillName.Alchemy) then
		modifier = 15;
	elseif(raceID == "Pandaren" and skill == SkillName.Cooking) then
		modifier = 15;
	end
	return SKILLCAP + modifier;
end


_G[ADDONNAME] = MyFuncs -- put our functions in the global environment under our addon name so we can access them from the .xml as ProfessionHUD.FunctionName